/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "filemanager.h"

// The constructor for the File Manager receives the strings for the source
// files, header files, other files, and the project location. It initializes
// the widgets and sets their properties. The dialog is then created and shown
// and the signals are connected to their widgets
FileManager::FileManager(Glib::ustring src, Glib::ustring hdr, 
                         Glib::ustring othr, Glib::ustring loc)
{
  // Setting source, header & other files and the location
  source_files = src;
  header_files = hdr;
  other_files = othr;
  src_location = Glib::ustring(loc + "src/");
  
  initialize_objects();  
   
  // Populate sources box & attach signals to the widgets
  on_radio_button_selected();
  sources->signal_clicked().connect(sigc::mem_fun(*this, &FileManager::on_radio_button_selected));
  others->signal_clicked().connect(sigc::mem_fun(*this, &FileManager::on_radio_button_selected));
  add->signal_clicked().connect(sigc::mem_fun(*this, &FileManager::on_add));
  del->signal_clicked().connect(sigc::mem_fun(*this, &FileManager::on_delete));
  close->signal_clicked().connect(sigc::mem_fun(*this, &FileManager::on_close));
}

Gtk::Dialog* FileManager::get_dialog()
{ return dialog;
}

// This destructor goes through all of the initialized widgets
// and deletes them to free up memory on close of the dialog
FileManager::~FileManager()
{
  delete add;
  delete del;
  delete close;
  delete files;
  delete sources;
  delete headers;
  delete others;
  delete vbox;
  delete hbox;
  delete hseparator;
  delete scrolled_win;
}

// One or more sources are added to the current type. The filenames(s) are then
// parsed into their base form and added to the list. The sources are then
// sorted and re-populated
void FileManager::on_add()
{
  Gtk::FileChooserDialog chooser("Choose File", Gtk::FILE_CHOOSER_ACTION_OPEN);
  chooser.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  chooser.add_button(Gtk::Stock::ADD, Gtk::RESPONSE_OK);
  chooser.set_current_folder(src_location);
  chooser.set_select_multiple(true);
  
  int result = chooser.run();
  switch (result)
  {
    // If OK is pressed
    case(Gtk::RESPONSE_OK):
    {
      int location;
      vector<Glib::ustring> filenames = chooser.get_filenames();
      
      // Go through the files & cut them into only the short filenames
      for (unsigned int i=0; i<filenames.size(); i++)
      {
        location = filenames[i].find_last_of("/");
        while (location != Glib::ustring::npos)
        {
          filenames[i].erase(0, location+1);
          location = filenames[i].find_last_of("/");
        }
        filenames[i] += "/";
      }
      
      // Add source, header or other file to the list
      for (unsigned int i=0; i<filenames.size(); i++)
      {
        if (sources->get_active())
          source_files += filenames[i];
        else if (headers->get_active())
          header_files += filenames[i];
        else
          other_files += filenames[i];
      }
      
      // Sort the list alphabetically & refresh it
      sort_source_files();
      on_radio_button_selected();
      break;
    }
    default:
      break; 
  }
}

// The currently selected file is deleted from the current TreeView. The
// list is then re-populated
void FileManager::on_delete()
{
  Glib::RefPtr<Gtk::TreeSelection> selection = files->get_selection();
  Gtk::ListStore::iterator iter = selection->get_selected();
  
  if (iter)
  {
    Gtk::TreeModel::Row row = *iter;
    Glib::ustring fn;
    int start;
    row->get_value(0, fn);
    
    // Delete the chosen source, header or other file's name
    if (sources->get_active())
    {
      start = source_files.find(fn, 0);
      source_files.erase(start, fn.length()+1);
    }
    else if (headers->get_active())
    {
      start = header_files.find(fn, 0);
      header_files.erase(start, fn.length()+1);
    }
    else
    {
      start = other_files.find(fn, 0);
      other_files.erase(start, fn.length()+1);
    }
  }
  
  // Refresh the list
  on_radio_button_selected();
}

// On the signal of the close button being clicked, a close response is emitted
void FileManager::on_close()
{
  // Write files to temp files to be parsed
  dialog->response(Gtk::RESPONSE_CLOSE);
}

// The list of sources, headers or other files is refreshed by this function.
// The sources are first removed from the list and then the new list is created.
void FileManager::on_radio_button_selected()
{
  Glib::ustring text;
  int start = 0;
  ModelColumns columns;
  Glib::RefPtr<Gtk::ListStore> model;
  Gtk::TreeModel::Row row;
  
  // Refresh files list
  files->remove_all_columns();
  model = Gtk::ListStore::create(columns);
  files->set_model(model);
  files->append_column("Project Files", columns.col_text);

  // Get whether to list sources for headers
  if (sources->get_active())
    text = source_files;
  else if (headers->get_active())
    text = header_files;
  else
    text = other_files;
  
  // Add the choice of files to the list
  for (unsigned int i=0; i<text.length(); i++)
  {
    if (text[i] == '/') 
    {
      row = *(model->append());
      row[columns.col_text] = text.substr(start, i-start);
      start = i+1;
    }
  }
}

// This takes the sources, headers and other files and sorts them alphabetically
// by parsing them into a vector, sorting them and then unparsing them.
void FileManager::sort_source_files()
{
  // Loop headers & sources
  for (unsigned int i=0; i<3; i++)
  {
    vector<Glib::ustring> names;
    Glib::ustring temp, str;
    int location;
    
    // Set the source, header or other file's names to the temp string
    if (i == 0) 
    { 
      temp = source_files;
      source_files = "\0";
    } 
    else if (i == 1) 
    {
      temp = header_files;
      header_files = "\0";
    }
    else
    {
      temp = other_files;
      other_files = "\0";
    }
    
    // Parse the files list
    location = temp.find("/",0);
    while (location != Glib::ustring::npos) 
    {
      names.push_back(temp.substr(0,location));
      temp.erase(0,location+1);
      location = temp.find("/",0);
    }
    
    // Sort files
    vector<Glib::ustring>::iterator start = names.begin();
    vector<Glib::ustring>::iterator end = names.end();
    sort(start,end);
    
    // Un-parse files list
    for (unsigned int x=0; x<names.size(); x++) 
    {
      if (i == 0)
        source_files += names.at(x) + "/";
      else if (i == 1)
        header_files += names.at(x) + "/";
      else
        other_files  += names.at(x) + "/";
    }     
  }
}

// All of the widgets for the File Manager dialog are initialized here
void FileManager::initialize_objects()
{
  Glib::ustring gladefile = string(INSTALL_PREFIX) + "/share/openldev/glade/projectfiles.glade";
  Glib::RefPtr<Gnome::Glade::Xml> xml = Gnome::Glade::Xml::create(gladefile);
  
  xml->get_widget("dialog", dialog);
  xml->get_widget("button_add", add);
  xml->get_widget("button_close", close);
  xml->get_widget("button_delete", del);
  xml->get_widget("treeview", files);
  xml->get_widget("radio_sources", sources);
  xml->get_widget("radio_headers", headers);
  xml->get_widget("radio_others", others);
}

// This allows the main openldev class to have access to the new lists off
// sources, headers and other files.
Glib::ustring FileManager::get_source_files()
{ return source_files; 
}

Glib::ustring FileManager::get_header_files()
{ return header_files; 
}

Glib::ustring FileManager::get_other_files()
{ return other_files; 
}
