/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "newfile.h"

// This is the default constructor for the new file dialog. It is meant
// to initialize all of the objects and set their properties. It then creates
// the dialog and displays it to the screen. It receives the project settings
// object so the location of project files can be accessed.
NewFileDialog::NewFileDialog (ProjectSettings *proj_settings)
{
  file_location = proj_settings->get_location() + "src/";
  
  initialize_objects();
  populate_combobox();
  
  apply->signal_clicked().connect(sigc::mem_fun(*this, &NewFileDialog::on_apply));
  cancel->signal_clicked().connect(sigc::mem_fun(*this, &NewFileDialog::on_cancel));
}

// The dialog and its widgets are initialized in this function.
void NewFileDialog::initialize_objects()
{
  Glib::ustring gladefile = string(INSTALL_PREFIX) + "/share/openldev/glade/newfile.glade";
  Glib::RefPtr<Gnome::Glade::Xml> xml = Gnome::Glade::Xml::create(gladefile);
  
  xml->get_widget("dialog", dialog);
  xml->get_widget("button_cancel", cancel);
  xml->get_widget("button_ok", apply);
  xml->get_widget("filetypes", combobox);
  xml->get_widget("filename", name);
}

Gtk::Dialog* NewFileDialog::get_dialog()
{ return dialog;
}

// This destructor goes through all of the initialized widgets
// and deletes them to free up memory on close of the dialog
NewFileDialog::~NewFileDialog()
{
  delete cancel;
  delete apply;
  delete name;
  delete combobox;
}

// When cancel is pressed, this function is called to emit a cancel signal
// and hide the dialog
void NewFileDialog::on_cancel(void)
{
  dialog->response(Gtk::RESPONSE_CANCEL);
  dialog->hide();
}

// When the apply button is pressed, it is checked that the file name only
// contains letters and numbers and that a version and location is specified.
// The file is then created in the project's source directory
void NewFileDialog::on_apply(void)
{
  Gtk::Entry *choice = combobox->get_entry();
  Glib::ustring choice_text = choice->get_text();
  
  Glib::ustring ntxt = name->get_text();
  
  // Create new file if no errors are found
  if (!(ntxt.find(" ",0) != Glib::ustring::npos || ntxt.empty()))
  {
    Glib::ustring ext1="~", ext2="~", ext3="~";
    file_location += name->get_text();
    
    // Create possible file extensions
    if (choice_text == "C Source File (.c)") {
      ext1 = ".c";
    } 
    else if (choice_text == "C Header File (.h)")  {
      ext1 = ".h";
    } 
    else if (choice_text == "C++ Header File (.hh)")  {
      ext1 = ".hh";
    }
    else if (choice_text == "C++ Source File (.cc)")  {
      ext1 = ".cc";
      ext2 = ".cpp";
      ext3 = ".cxx";
    }
    
    // Add extension if it is not there
    if (name->get_text().find(ext1,0) == Glib::ustring::npos && 
        name->get_text().find(ext2,0) == Glib::ustring::npos && 
        name->get_text().find(ext3,0) == Glib::ustring::npos)
      if (ext1 != "~")
        file_location += ext1;
    
    // Write new file
    write_file(file_location, "");
    
    dialog->response(Gtk::RESPONSE_ACCEPT);
    dialog->hide();
  }
  // Throw an error
  else 
    error("You must specify a\nfile name!");
}

// This adds the different types of files to the combobox
void NewFileDialog::populate_combobox()
{
  list<Glib::ustring> types;
  
  types.push_back("C Source File (.c)");
  types.push_back("C Header File (.h)");
  types.push_back("C++ Header File (.hh)");
  types.push_back("C++ Source File (.cc)");
  types.push_back("Other File");

  combobox->set_popdown_strings(types);
}

// This allows the main openldev class access to the filename of the file.
Glib::ustring NewFileDialog::get_filename() 
{ return file_location; 
}
