/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "notebookmanager.h"
#include "syntax.h"

#include <libgnomevfsmm/init.h>
#include <libgnomevfsmm/uri.h>

// Notebook manager constructor that creates the notebook and sets its
// properties. It also sets the number of tabs to zero
NotebookManager::NotebookManager()
{
  // Create the notebook & set its settings
	set_show_tabs(true);
  set_tab_pos(Gtk::POS_TOP);
  set_scrollable(true);
	show();
  signal_event_after().connect(sigc::mem_fun(*this, &NotebookManager::notebook_event));
  tab_count = 0;
}

// Catch all events of the notebook so that the focus of the textview can
// be restored when a notebook tab is clicked
void NotebookManager::notebook_event(GdkEvent* ev)
{
  if (ev->type == GDK_BUTTON_PRESS)
    tabs[get_current_page()].textview->grab_focus();
}

// This function allows the main openldev class direct access to
// the notebook widget
Gtk::Notebook* NotebookManager::get_notebook()
{ return this;
}

// Add a tab to the notebook. It receives the filename that is to be opened
// and the environment settings object. A new tab object is created if the
// file is not already open. The signals are then connected to the widgets and
// the fonts are set to the line numbering & TextView
bool NotebookManager::add_tab(Glib::ustring filename, EnvironmentSettings *env)
{
  NotebookTab tab;
  tab.fn = filename;
  
  // Parse the filename to remove directory structure
  while (filename.find('/',0) != string::npos)
    filename.erase(0,filename.find('/',0)+1);

  // Check to make sure file is not already open
  for (int i=0; i<tab_count; i++)
  {    
    if (tab.fn == tabs[i].fn)
    {
      set_current_page(i);
      return false;
    }
  }

  // Create temporary tab & initialize the objects
	tab.initialize_objects(filename);
	tab.tab_id = tab_count;

  // Add the tab to the notebook and notebook_tab's deque
	tabs.push_back(tab);
	append_page(*tabs[tab_count].swin, *tabs[tab_count].hbox);
  
  // Set all of the signal commands
	tabs[tab_count].button->signal_clicked().connect(sigc::bind<Glib::ustring>(sigc::mem_fun(*this, 
          &NotebookManager::delete_tab), tabs[tab_count].fn));
  tabs[tab_count].textview->signal_key_press_event().connect(sigc::mem_fun(*this,
          &NotebookManager::on_keypress), false);
   
  // Set the font and tab size of the textview  
  Glib::ustring font_description = env->get_font_face() + " " + env->get_font_size();
  Pango::FontDescription fd(font_description);  
  tabs[tab_count].textview->modify_font(fd);
  tabs[tab_count].textview->set_tabs(get_tab_array(fd, env, tabs[tab_count].textview));
  
  // Do final initialization
  set_current_page(tab_count);
	tab_count++;
  return true;
}

// Revert to the saved version of the file. The file must be set as unmodified,
// then the file is reread & restored to its original version.
void NotebookManager::revert_to_saved()
{
  mark_current_tab_modified(false);
  
  int current = get_current_page();
  Glib::ustring text = read_file(tabs[current].fn);
  
  Glib::RefPtr<Gtk::SourceBuffer> buffer = tabs[current].textview->get_buffer();
  buffer->set_text(text);
  tabs[current].textview->set_buffer(buffer);
}

// Delete the currently selected tab. First the user must confirm a closing of
// the file if it is modified. Then find the close button that was clicked,
// remove the tab, and clean up after the tab. It receives the full filename
// of the file that is going to be closed.
void NotebookManager::delete_tab(Glib::ustring fn)
{
  if (tabs[get_current_page()].label->get_text().find('*', 0) != string::npos)
    if (!question("The file you are closing is not saved.\nAre you sure you want to close?"))
      return;

  // Find file button that was clicked & get buffer
  int tab = 0;
  while (fn != tabs[tab].fn)
    tab++;

  // Remove the tab
	deque<NotebookTab>::iterator itr = tabs.begin();
	remove_page(tab);

  // Move the iter to the current tab
	for (int i=0; i < tab; i++)
		itr++;
	
  // Remove the tab objects & information
  tab_count--;
	tabs.erase(itr);
  
  // Set the current page to the previous one if there is a tab open
  if (tab_count != 0)
    set_current_page(tab_count);
}

// Return whether there is currently a file open. This will prevent a
// segmentation fault when compiling
bool NotebookManager::get_file_open()
{
  if (get_n_pages() > 0)
    return true;
  
  return false;
}

// Save all files in the tabs. Loop through all of the files and set them as
// unmodified. Then write the text from the buffer to the tab's full filename.
void NotebookManager::save_all_files()
{
  for (int i=0; i<get_n_pages(); i++)
  {    
    // Set the label as unmodified
    Glib::ustring text = tabs[i].label->get_text();
    if (text.find('*', 0) != string::npos)
    {
      text.erase(text.find('*', 0), 1);
      tabs[i].label->set_text(text);
    
      // Save the file if it is unmodified
      write_file(tabs[i].fn, tabs[i].textview->get_buffer()->get_text(true));
    }
  }
}

// Destroy everything in the notebook manager. This will remove all of the
// pages until the there are no more left.
void NotebookManager::erase_all_tabs()
{
  // Remove all of the tabs and tab information
  tabs.erase(tabs.begin(), tabs.end());
  while (get_n_pages() != 0)
    remove_page(0);
  
  tab_count = 0;
}

// Return the current tab so that it can be edited by the main openldev class
int NotebookManager::get_current_tab()
{ return get_current_page();
}

bool NotebookManager::on_keypress(GdkEventKey *ev)
{
  if (ev->type == GDK_KEY_PRESS) 
    mark_current_tab_modified(true);
    
  return false;
}

// Mark the current file as modified or unmodified. It receives whether it should
// be marked as modified or unmodified. Modified is shown by adding an asterick
// to the end of the short filename in the tab header
void NotebookManager::mark_current_tab_modified(bool modified)
{
  int current = get_current_page();
  Glib::ustring text = tabs[current].label->get_text();
  
  // If the file was modified
  if (modified)
  {
    if (text.find('*', 0) == string::npos)
      text += "*";
  }
  // If it was not modified
  else 
  {
    if (text.find('*', 0) != string::npos)
      text.erase(text.find('*', 0), 1);
  }
  
  tabs[current].label->set_text(text);
}


// Modify the font of each textview & line numbering. This receives a font
// description and is used when the user changes their default font.
void NotebookManager::modify_fonts(Pango::FontDescription fd)
{
  // Foreach tab, modify the fonts
  for (int i=0; i<tab_count; i++)
    tabs[i].textview->modify_font(fd);
}

// Scroll to the current cursor position. This is used for keypresses, find
// commands and goto line commands. It is needed for keypresses because the line
// numbering messes up the default properties of the TextView such as scrolling
// to the cursor when it reaches the end of the screen.
void NotebookManager::scroll_to_mark()
{
  int current = get_current_page();
  tabs[current].textview->scroll_to(tabs[current].textview->get_buffer()->get_insert(), 0.0);
}

// Get the filename of the current tab. This returns the full filename.
Glib::ustring NotebookManager::get_current_filename()
{ return tabs[get_current_page()].fn;
}

// Get the short filename of the current tab. The asterick must be removed from
// the end of modified file names.
Glib::ustring NotebookManager::get_current_short_filename()
{
  int current = get_current_page();
  Glib::ustring fn = tabs[current].label->get_text();
  if (fn.find("*", 0) != string::npos)
    fn.erase(fn.find("*", 0), 1);
  
  return fn;
}

// Return the current textview so the main openldev class can directly access
// it. If there are no files open, then return a brand-spanking new textview
// that can just be trashed.
Gtk::SourceView* NotebookManager::get_textview()
{
  int current = get_current_page();

  // Return the current textview
  if (tab_count != 0)
    return tabs[current].textview;
  // Return an empty textview if there are none
  else
  {
    Gtk::SourceView *textview = new Gtk::SourceView();
    return textview;
  }
}

// Return the current buffer so the main openldev class can directly access it. 
Glib::RefPtr<Gtk::SourceBuffer> NotebookManager::get_textbuffer()
{ return get_textview()->get_buffer();
}

// Return the offset of the cursor iterator
int NotebookManager::get_insert_offset()
{ return get_textbuffer()->get_insert()->get_iter().get_offset();
}

// Return the offset of the selection bound iterator
int NotebookManager::get_selection_offset()
{ return get_textbuffer()->get_selection_bound()->get_iter().get_offset();
}

Glib::ustring get_mime_type(Glib::ustring fn)
{
  Gnome::Vfs::init();
  Glib::RefPtr<Gnome::Vfs::Uri> uri = Gnome::Vfs::Uri::create(fn);
  Glib::RefPtr<Gnome::Vfs::FileInfo> fileinfo = uri->get_file_info(Gnome::Vfs::FILE_INFO_GET_MIME_TYPE);

  return fileinfo->get_mime_type();
}

void NotebookManager::set_properties_one_tab(int tab, EnvironmentSettings* env)
{
  tabs[tab].textview->set_show_line_numbers(env->get_line_numbers());
  tabs[tab].textview->set_tabs_width(env->get_tabsize());
  tabs[tab].textview->set_auto_indent(env->get_auto_indent());
  tabs[tab].textview->set_show_margin(env->get_rightmargin());
  tabs[tab].textview->set_insert_spaces_instead_of_tabs(env->get_replace_tabs());
  tabs[tab].textview->get_buffer()->set_max_undo_levels(env->get_undo());
  tabs[tab].textview->get_buffer()->set_check_brackets(env->get_brackets());
   
  Glib::ustring mime_type = get_mime_type(tabs[tab].fn); 
  if (mime_type == "NONE")
    tabs[tab].textview->get_buffer()->set_highlight(false);
  else
  {
    tabs[tab].textview->get_buffer()->set_highlight(true);
    Gtk::SourceLanguage* language = tabs[tab].textview->get_buffer()->get_languages_manager()->get_language_from_mime_type(mime_type);

    if (language) 
      tabs[tab].textview->get_buffer()->set_language(language);
  }
  
  if (env->get_wrap())
  {
    if (env->get_wrapsplit())
      tabs[tab].textview->set_wrap_mode(Gtk::WRAP_CHAR);
    else
      tabs[tab].textview->set_wrap_mode(Gtk::WRAP_WORD);
  }
  else
    tabs[tab].textview->set_wrap_mode(Gtk::WRAP_NONE);
}

void NotebookManager::set_properties (bool all_files, EnvironmentSettings* env)
{
  if (all_files)
    for (int i=0; i<tab_count; i++)
      set_properties_one_tab(i, env);
  else
    set_properties_one_tab(get_current_page(), env);
}

void NotebookManager::set_filename (Glib::ustring file)
{
  while (file.find('/',0) != string::npos)
    file.erase(0, file.find('/',0) + 1);
  tabs[get_current_page()].label->set_text(file);
}
